/*
LDAPEOAdaptor.framework - Providing EOF with an interface for LDAP
Copyright (C) 1998  Ryan Krebs and Stanford University

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Comments, suggestions, bug reports, etc can be directed to:
rkrebs@cisnet.stanford.edu
*/

#import "LDAPAdaptorPrivate.h"

// This is all standard EOAdaptor stuff
@implementation LDAPAdaptor
// This is pretty self explanatory
- (EOAdaptorContext *)createAdaptorContext
{
	return [[[LDAPContext alloc] initWithAdaptor:self] autorelease];
}

// Here we check to make sure the type isn't something Binary
- (BOOL)isValidQualifierType:(NSString *)typeName model:(EOModel *)model
{
	return ![typeName isEqualToString:@"Data"];
}

// Open and Close a channel to ensure the dictionary works
- (void)assertConnectionDictionaryIsValid
{
	if (![self hasOpenChannels]) {
		NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		LDAPContext *context = [self createAdaptorContext];
		LDAPChannel *channel = [(LDAPContext *)context createAdaptorChannel];

		[(LDAPChannel *)channel openChannel];
		[(LDAPChannel *)channel closeChannel];

		[pool release];
	}
}

// nil, not using SQLExpression here
// maybe if I knew anything about SQL.....
- (Class)defaultExpressionClass
{
	return nil;
}

/////////////////////////
// External Type Mapping
/////////////////////////

/* The external types are String and Data, by default matching
    NSString and NSData.  I thought about adding things like Image
    or Number, but LDAP uses strings to store anything that isn't
    binary data, so numbers would be pointless, and they just
    complicate things.  */
+ (NSString *)internalTypeForExternalType:(NSString *)extType model:(EOModel *)model
{
	NSString *intType;

	if ([extType isEqualToString:@"Data"])
		intType = [NSString stringWithString:@"NSData"];
	else
		intType = [NSString stringWithString:@"NSString"];

	return intType;
}

// String and Data
+ (NSArray *)externalTypesWithModel:(EOModel *)model
{
	return [NSArray arrayWithObjects:@"Data", @"String", nil];
}

// Here we give it an external type
+ (void)assignExternalInfoForAttribute:(EOAttribute *)attribute
{
	if ([attribute adaptorValueType] == EOAdaptorBytesType) {
		[attribute setExternalType:@"Data"];
	} else {
		[attribute setExternalType:@"String"];
	}
}
@end
