/*
LDAPEOAdaptor.framework - Providing EOF with an interface for LDAP
Copyright (C) 1998  Ryan Krebs and Stanford University

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Comments, suggestions, bug reports, etc can be directed to:
rkrebs@cisnet.stanford.edu
*/

#import <EOAccess/EOAccess.h>

@interface LDAPController : NSObject
{
	LDAP		*_connection;
	LDAPMessage	*_result;
	NSString	*_serverName, *_userDN,
			*_searchBase, *_password,
			*_valueSeparator;
	int		_serverPort,  _timeout,
			_searchScope, _authMethod,
			_errorNum;
	BOOL		_attributesOnly, _shouldRaise;
	NSArray		*_attributes, *_eoAttributes;
}

+ (LDAPController *)controllerWithConnectionDictionary:(NSDictionary *)dictionary;
- (void)initWithConnectionDictionary:(NSDictionary *)dictionary;

//  Put in lots of set and get methods here
- (void)setAttributesToFetch:(NSArray *)attributes;
- (NSArray *)attributesToFetch;

- (void)setAttributesOnly:(BOOL)yn;
- (BOOL)attributesOnly;

- (void)setSearchBase:(NSString *)base;
- (NSString *)searchBase;

- (void)setSearchScope:(int)scope;
- (int)searchScope;

- (void)setValueSeparator:(NSString *)valSep;
- (NSString *)valueSeparator;

- (void)setTimeout:(int)timeout;
- (int)timeout;

- (void)setUserDN:(NSString *)userDN password:(NSString *)password;

- (LDAP *)connection;

- (unsigned)autoconnect;
- (unsigned)connect;
- (unsigned)disconnect;

- (void)dealloc;

- (BOOL)shouldRaise;
- (void)setShouldRaise:(BOOL)raise;
- (NSString *)errorString;
- (NSString *)errorStringWithErrorNum:(int)errorNum;
@end

@interface LDAPController (AsynchOperations)
- (unsigned)initiateSearchWithFilter:(NSString *)filter;
- (int)retrieveResultsForID:(int)msgID all:(BOOL)all message:(LDAPMessage **)message;
- (int)retrieveResultsForID:(int)msgID;
- (unsigned)abandonOperationWithID:(int)msgID;
@end

@interface LDAPController (SynchOperations)
- (NSArray *)searchWithFilter:(NSString *)filter;
- (unsigned)deleteEntry:(NSString *)dn;
- (unsigned)deleteEntriesMatching:(EOQualifier *)qualifier;
- (unsigned)addEntry:(NSString *)dn withDictionary:(NSDictionary *)dictionary;
- (unsigned)addEntry:(NSString *)dn withMods:(LDAPMod **)mods;
- (unsigned)updateEntry:(NSString *)dn withDictionary:(NSDictionary *)dictionary;
- (unsigned)updateEntry:(NSString *)dn withMods:(LDAPMod **)mods;
- (unsigned)updateEntriesMatching:(EOQualifier *)qualifier withDictionary:(NSDictionary *)dictionary;
- (unsigned)updateEntriesMatching:(EOQualifier *)qualifier withMods:(LDAPMod **)mods;
- (NSArray *)findEntriesMatching:(EOQualifier *)qualifier;
- (unsigned)renameEntry:(NSString *)dn withNewRDN:(NSString *)newRDN;
@end

@interface LDAPController (Conversions)
- (NSString *)parentOfEntryWithName:(NSString *)dn;
- (NSString *)rdnOfEntryWithName:(NSString *)dn;
- (NSMutableDictionary *)dictionaryWithEntry:(LDAPMessage *)entry;
- (NSArray *)dictionaryArrayWithEntries:(LDAPMessage *)entries;
- (LDAPMod *)modificationForAttribute:(NSString *)attribute values:(NSArray *)values;
- (LDAPMod **)modificationsWithDictionary:(NSDictionary *)dictionary;
+ (void)freeMod:(LDAPMod *)mod;
+ (void)freeMods:(LDAPMod **)mods;
@end