/*
LDAPEOAdaptor.framework - Providing EOF with an interface for LDAP
Copyright (C) 1998  Ryan Krebs and Stanford University

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Comments, suggestions, bug reports, etc can be directed to:
rkrebs@cisnet.stanford.edu
*/

#import "LDAPAdaptor.h"
#import "LDAPLoginPanel.h"

#import <Appkit/Appkit.h>

@implementation LDAPLoginPanel

- init
{
	[super init];
	[NSBundle loadNibNamed:@"LDAPLoginPanel" owner:self];

	return self;
}

- (NSDictionary *)runPanelForAdaptor:(EOAdaptor *)adaptor validate:(BOOL)yn
{
	NSDictionary *result;
	BOOL success;

	[self setConnectionDictionary:[adaptor connectionDictionary]];
	do {
		if ([NSApp runModalForWindow:[serverNameField window]] != NSRunContinuesResponse) {
			result = nil;
			[[serverNameField window] orderOut:nil];
			break;
		}
		result = [self connectionDictionary];
		[[serverNameField window] orderOut:nil];

		if (!yn) break;

		success = YES;
		[adaptor setConnectionDictionary:result];
		NS_DURING
			[adaptor assertConnectionDictionaryIsValid];
		NS_HANDLER
			NSRunAlertPanel (NULL, @"%@", NULL, NULL, NULL, [localException reason]);
			[self setConnectionDictionary:[adaptor connectionDictionary]];
			success = NO;
		NS_ENDHANDLER
	} while (!success);
	
	return result;
}

- (NSDictionary *)connectionDictionary
{
	NSMutableDictionary *info;
	NSString *serverName, *serverPort, *userDN, *password, *defaultBase, *schemaBase, *valueSeparator, *timeout, *authMethod;

	info = [NSMutableDictionary dictionaryWithCapacity:9];

	serverName = [serverNameField stringValue];
	if (!serverName)
		serverName = @"";
	[info setObject:serverName forKey:serverNameKey];

        serverPort = [serverPortField stringValue];
        if (!serverPort)
        	serverPort = @"389";
        [info setObject:serverPort forKey:serverPortKey];

        userDN = [userDNField stringValue];
       	if (!userDN)
        	userDN = @"";
        [info setObject:userDN forKey:userDNKey];

	password = [passwordField stringValue];
	if (!password)
		password = @"";
	[info setObject:password forKey:passwordKey];

	defaultBase = [defaultBaseField stringValue];
	if (!defaultBase)
		defaultBase = @"";
	[info setObject:defaultBase forKey:defaultBaseKey];

	schemaBase = [schemaBaseField stringValue];
	if (!schemaBase)
		schemaBase = @"cn=schema";
	[info setObject:schemaBase forKey:schemaBaseKey];

	valueSeparator = [valueSeparatorField stringValue];
	if (!valueSeparator)
		valueSeparator = @" | ";
	[info setObject:valueSeparator forKey:valueSeparatorKey];	

	timeout = [timeoutField stringValue];
	if (!timeout)
		timeout = @"0";
	[info setObject:timeout forKey:timeoutKey];

	authMethod = [authenticationMethodField titleOfSelectedItem];
	if ([authMethod isEqualToString:@"Simple"])
		[info setObject:@"LDAP_AUTH_SIMPLE" forKey:authenticationMethodKey];

	return info;
}

- (void)setConnectionDictionary:(NSDictionary *)connectionInfo
{
	NSString *serverName, *serverPort, *userDN, *password, *defaultBase, *schemaBase, *valueSeparator, *timeout, *authMethod;

	serverName = [connectionInfo objectForKey:serverNameKey];
	if (!serverName)
		serverName = @"";
	[serverNameField setStringValue:serverName];

	serverPort = [connectionInfo objectForKey:serverPortKey];
	if (!serverPort)
		serverPort = @"389";
	[serverPortField setStringValue:serverPort];

        userDN = [connectionInfo objectForKey:userDNKey];
        if (!userDN)
        	userDN = @"";
        [userDNField setStringValue:userDN];

        password = [connectionInfo objectForKey:passwordKey];
        if (!password)
        	password = @"";
        [passwordField setStringValue:password];

        defaultBase = [connectionInfo objectForKey:defaultBaseKey];
        if (!defaultBase)
                defaultBase = @"";
        [defaultBaseField setStringValue:defaultBase];

	schemaBase = [connectionInfo objectForKey:schemaBaseKey];
	if (!schemaBase)
		schemaBase = @"cn=schema";
	[schemaBaseField setStringValue:schemaBase];

	valueSeparator = [connectionInfo objectForKey:valueSeparatorKey];
	if (!valueSeparator)
		valueSeparator = @" | ";
	[valueSeparatorField setStringValue:valueSeparator];

	timeout = [connectionInfo objectForKey:timeoutKey];
	if (!timeout)
		timeout = @"0";
	[timeoutField setStringValue:timeout];

// FIXME - probably should have the Kerberos item active when
//  kerberos authentication is available
	authMethod = [connectionInfo objectForKey:authenticationMethodKey];
	if ([authMethod isEqualToString:@"LDAP_AUTH_SIMPLE"] || !authMethod)
		[authenticationMethodField selectItemWithTitle:@"Simple"];
}

- (void)cancel:sender
{
	[NSApp stopModalWithCode:NSRunAbortedResponse];
}

- (void)login:sender
{
	[NSApp stopModalWithCode:NSRunContinuesResponse];
}

@end
